@@ -10,22 +10,23 @@
 /*                                                                         */
 /* *********************************************************************** */
 /*                                                                         */
-/* Copyright (C) 2001  Yves Renard.                                        */
+/* Copyright (C) 1995-2002  Yves Renard.                                   */
 /*                                                                         */
 /* This file is a part of GETFEM++                                         */
 /*                                                                         */
 /* This program is free software; you can redistribute it and/or modify    */
-/* it under the terms of the GNU General Public License as published by    */
-/* the Free Software Foundation; version 2 of the License.                 */
+/* it under the terms of the GNU Lesser General Public License as          */
+/* published by the Free Software Foundation; version 2.1 of the License.  */
 /*                                                                         */
 /* This program is distributed in the hope that it will be useful,         */
 /* but WITHOUT ANY WARRANTY; without even the implied warranty of          */
 /* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
-/* GNU General Public License for more details.                            */
+/* GNU Lesser General Public License for more details.                     */
 /*                                                                         */
-/* You should have received a copy of the GNU General Public License       */
-/* along with this program; if not, write to the Free Software Foundation, */
-/* Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.         */
+/* You should have received a copy of the GNU Lesser General Public        */
+/* License along with this program; if not, write to the Free Software     */
+/* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,  */
+/* USA.                                                                    */
 /*                                                                         */
 /* *********************************************************************** */
 
@@ -54,16 +55,16 @@ namespace bgeot
       inline const T& operator ()(size_type l, size_type c) const
       {
         #ifdef __GETFEM_VERIFY
-	if (l >= N || c >= N) out_of_range_error();
+	if (l >= N || c >= N) this->out_of_range_error();
         #endif
-	return *(begin() + c*N+l);
+	return *(this->begin() + c*N+l);
       }
       inline T& operator ()(size_type l, size_type c)
       {
         #ifdef __GETFEM_VERIFY
-	if (l >= N || c >= N) out_of_range_error();
+	if (l >= N || c >= N) this->out_of_range_error();
         #endif
-        return *(begin() + c*N+l);
+        return *(this->begin() + c*N+l);
       }
 
       void clear_line(size_type i);
@@ -86,17 +87,17 @@ namespace bgeot
   template<class T, int N> void fsmatrix<T,N>::fill(T a, T b)
   { 
     fsvector<T,N*N>::fill(b);
-    iterator p = begin(), e = end();
+    iterator p = this->begin(), e = this->end();
     while (p < e) { *p = a; p += N+1; }
   }
 
   template<class T, int N> void fsmatrix<T,N>::clear_line(size_type i)
-  { iterator p = begin()+i, e = end(); while (p < e) { *p = T(0); p += N; } }
+  { iterator p = this->begin()+i, e = this->end(); while (p < e) { *p = T(0); p += N; } }
 
   template<class T, int N>
     void fsmatrix<T,N>::line_exchange(size_type i, size_type j)
   {
-    iterator p1 = begin(), e = end(), p2 = p1; p1 += i; p2 += j;
+    iterator p1 = this->begin(), e = this->end(), p2 = p1; p1 += i; p2 += j;
     while (p1 < e) { std::swap(*p1, *p2); p1 += N; p2 += N; }
   }
 
@@ -104,23 +105,23 @@ namespace bgeot
   {
     for (size_type l = 0; l < N; l++)
     {
-      iterator p1 = begin()+l+l*N, e = end(), p2 = p1; p1 += N; p2++;
+      iterator p1 = this->begin()+l+l*N, e = this->end(), p2 = p1; p1 += N; p2++;
       while (p1 < e) { std::swap(*p1, *p2); p1 += N; p2++; }
     }
   }
 
   template<class T, int N> void fsmatrix<T,N>::l_mul(size_type i, const T &x)
-  { iterator p = begin()+i, e = end(); while (p1 < e) { *p *= x; p += N; } }
+  { iterator p = this->begin()+i, e = this->end(); while (p < e) { *p *= x; p += N; } }
 
   template<class T, int N>
     fsmatrix<T,N>& fsmatrix<T,N>::operator *=(const fsmatrix<T,N>& m)
   {
-    if (begin() != m.begin())
+    if (this->begin() != m.begin())
     {
       fsvector<T,N> tmp;
       for (size_type l = 0; l < N; l++)
       {
-	iterator p1 = begin()+l, e = end(), p2 = tmp.begin(), te = tmp.end();
+	iterator p1 = this->begin()+l, e = this->end(), p2 = tmp.begin(), te = tmp.end();
 	const_iterator p3 = p1;
 	while (p3 < e) { *p2++ = *p3; p3 += N; }
 	for (size_type c = 0; c < N; c++)
@@ -137,8 +138,8 @@ namespace bgeot
       fsmatrix<T,N> q = *this;
       for (size_type l = 0; l < N; l++)
       {
-	iterator p1 = begin() + l, e = q.end(), p2, p3;
-	for (c = 0; c < N; c++)
+	iterator p1 = this->begin() + l, e = q.end(), p2, p3;
+	for (size_type c = 0; c < N; c++)
 	{
 	  *p1 = T(0);
 	  p2 = q.begin()+l; p3 = q.begin() + N * c;
@@ -223,20 +224,20 @@ namespace bgeot
       inline const T& operator ()(size_type l, size_type c) const
       {
         #ifdef __GETFEM_VERIFY
-	if (l >= nbl || c >= nbc) out_of_range_error();
+	if (l >= nbl || c >= nbc) this->out_of_range_error();
         #endif
-	return *(begin() + c*nbl+l);
+	return *(this->begin() + c*nbl+l);
       }
       inline T& operator ()(size_type l, size_type c)
       {
         #ifdef __GETFEM_VERIFY
-	if (l >= nbl || c >= nbc) out_of_range_error();
+	if (l >= nbl || c >= nbc) this->out_of_range_error();
         #endif
-	return *(begin() + c*nbl+l);
+	return *(this->begin() + c*nbl+l);
       }
 
       void resize(size_type l, size_type c)
-      { if (c * l != size()) vsvector<T>::resize(c*l); nbl = l; nbc = c; }
+      { if (c * l != this->size()) vsvector<T>::resize(c*l); nbl = l; nbc = c; }
       void clear_line(size_type i); 
       void line_exchange(size_type, size_type);
       void transpose(void);
@@ -268,16 +269,16 @@ namespace bgeot
   template<class T> void vsmatrix<T>::fill(T a, T b)
   { 
     vsvector<T>::fill(b);
-    iterator p = begin(), e = end();
+    iterator p = this->begin(), e = this->end();
     while (p < e) { *p = a; p += nbl+1; }
   }
 
   template<class T> void vsmatrix<T>::clear_line(size_type i)
-  { iterator p = begin()+i, e = end(); while (p < e) { *p = T(0); p += nbl; } }
+  { iterator p = this->begin()+i, e = this->end(); while (p < e) { *p = T(0); p += nbl; } }
 
   template<class T> void vsmatrix<T>::line_exchange(size_type i, size_type j)
   {
-    iterator p1 = begin(), e = end(), p2 = p1; p1 += i; p2 += j;
+    iterator p1 = this->begin(), e = this->end(), p2 = p1; p1 += i; p2 += j;
     while (p1 < e) { std::swap(*p1, *p2); p1 += nbl; p2 += nbl; }
   }
 
@@ -287,7 +288,7 @@ namespace bgeot
     {
       for (size_type l = 0; l < this->nbl; l++)
       {
-	iterator p1 = begin()+l+l*nbl, e = end(), p2 = p1; p1 += nbl; p2++;
+	iterator p1 = this->begin()+l+l*nbl, e = this->end(), p2 = p1; p1 += nbl; p2++;
 	while (p1 < e) { std::swap(*p1, *p2); p1 += nbl; p2++; }
       }
     }
@@ -302,19 +303,19 @@ namespace bgeot
   }
 
   template<class T> void vsmatrix<T>::l_mul(size_type i, const T &x)
-  { iterator p = begin()+i, e = end(); while (p1 < e) { *p *= x; p += nbl; } }
+  { iterator p = this->begin()+i, e = this->end(); while (p < e) { *p *= x; p += nbl; } }
 
   template<class T>  vsmatrix<T>& vsmatrix<T>::operator *=(const vsmatrix<T>& m)
   {
     if (nbc != m.nbl || nbc != m.nbc)
       DAL_THROW(dimension_error, "dimensions mismatch");
     
-    if (begin() != m.begin())
+    if (this->begin() != m.begin())
     {
       vsvector<T> tmp(nbc);
       for (size_type l = 0; l < nbl; l++)
       {
-	iterator p1 = begin()+l, e = end(), p2 = tmp.begin(), te = tmp.end();
+	iterator p1 = this->begin()+l, e = this->end(), p2 = tmp.begin(), te = tmp.end();
 	const_iterator p3 = p1;
 	while (p3 < e) { *p2++ = *p3; p3 += nbl; }
 	for (size_type c = 0; c < nbc; c++)
@@ -331,7 +332,7 @@ namespace bgeot
       vsmatrix<T> q = *this;
       for (size_type l = 0; l < nbl; l++)
       {
-	iterator p1 = begin() + l, e = q.end(), p2, p3;
+	iterator p1 = this->begin() + l, e = q.end(), p2, p3;
 	for (size_type c = 0; c < nbc; c++)
 	{
 	  *p1 = T(0);
@@ -395,8 +396,8 @@ namespace bgeot
     typename vsmatrix<T>::iterator p1 = res.begin();
     typename vsmatrix<T>::const_iterator p2, p3, e;
     
-    for (typename vsmatrix<T>::size_type c = 0; c < p.ncols(); c++)
-      for (typename vsmatrix<T>::size_type l = 0; l < p.nrows(); l++, p1++)
+    for (typename vsmatrix<T>::size_type c = 0; c < res.ncols(); c++)
+      for (typename vsmatrix<T>::size_type l = 0; l < res.nrows(); l++, p1++)
       {
 	*p1 = T(0);
 	p2 = m.begin() + l; e = m.end(); p3 = n.begin() + c * n.nrows();
@@ -552,8 +553,9 @@ namespace bgeot
   { // mr = m1 * m2; optimisable.
    
     if (m1.ncols() != m2.nrows() || mr.nrows() != m1.nrows()
-	  || mr.ncols() != m2.ncols() || ((void *)(&mr) == (void *)(&m1))
-	  || ((void *)(&mr) == (void *)(&m2)))
+	  || mr.ncols() != m2.ncols() 
+	  || ((const void *)(&mr) == (const void *)(&m1))
+	  || ((const void *)(&mr) == (const void *)(&m2)))
       DAL_THROW(dimension_error, "dimensions mismatch");
 
     for (size_t i = 0; i < mr.nrows(); ++i)
@@ -565,7 +567,7 @@ namespace bgeot
     void mat_vect_product(const MAT &m, const VECT &v, VECTR &vr)
   { // vr = m * v; optimisable.
     if (m.ncols() != v.size() || vr.size() != m.nrows()
-	|| ((void *)(&vr) == (void *)(&v)))
+	|| ((const void *)(&vr) == (const void *)(&v)))
       DAL_THROW(dimension_error, "dimensions mismatch");
     
     for (size_t i = 0; i < vr.size(); ++i) vr[i] = lv_product(m, i, v);
@@ -575,8 +577,9 @@ namespace bgeot
     void mat_product_tt(const MAT1 &m1, const MAT2 &m2, MATR &mr) 
   { // mr = transp(m1) * trans(m2); optimisable.
     if (m1.nrows() != m2.ncols() || mr.nrows() != m1.ncols()
-	  || mr.ncols() != m2.nrows() || ((void *)(&mr) == (void *)(&m1))
-	  || ((void *)(&mr) == (void *)(&m2)))
+	  || mr.ncols() != m2.nrows()
+	  || ((const void *)(&mr) == (const void *)(&m1))
+	  || ((const void *)(&mr) == (const void *)(&m2)))
       DAL_THROW(dimension_error, "dimensions mismatch");
 
     for (size_t i = 0; i < mr.nrows(); ++i)
@@ -588,7 +591,7 @@ namespace bgeot
     void mat_vect_product_t(const MAT &m, const VECT &v, VECTR &vr)
   { // vr = transp(m) * v; optimisable.
     if (m.ncols() != vr.size() || v.size() != m.nrows()
-	|| ((void *)(&vr) == (void *)(&v)))
+	|| ((const void *)(&vr) == (const void *)(&v)))
       DAL_THROW(dimension_error, "dimensions mismatch");
   
     for (size_t i = 0; i < vr.size(); ++i) vr[i] = cv_product(m, i, v);
@@ -598,8 +601,9 @@ namespace bgeot
     void mat_product_tn(const MAT1 &m1, const MAT2 &m2, MATR &mr) 
   { // mr = trans(m1) * m2; optimisable.
     if (m1.nrows() != m2.nrows() || mr.nrows() != m1.ncols()
-	  || mr.ncols() != m2.ncols() || ((void *)(&mr) == (void *)(&m1))
-	  || ((void *)(&mr) == (void *)(&m2)))
+	  || mr.ncols() != m2.ncols()
+	  || ((const void *)(&mr) == (const void *)(&m1))
+	  || ((const void *)(&mr) == (const void *)(&m2)))
       DAL_THROW(dimension_error, "dimensions mismatch");
     
     for (size_t i = 0; i < mr.nrows(); ++i)
@@ -611,8 +615,9 @@ namespace bgeot
     void mat_add_product_tn(const MAT1 &m1, const MAT2 &m2, MATR &mr) 
   { // mr += trans(m1) * m2; optimisable.
     if (m1.nrows() != m2.nrows() || mr.nrows() != m1.nrows()
-	  || mr.ncols() != m2.ncols() || ((void *)(&mr) == (void *)(&m1))
-	  || ((void *)(&mr) == (void *)(&m2)))
+	  || mr.ncols() != m2.ncols()
+	  || ((const void *)(&mr) == (const void *)(&m1))
+	  || ((const void *)(&mr) == (const void *)(&m2)))
       DAL_THROW(dimension_error, "dimensions mismatch");
 
     for (size_t i = 0; i < mr.nrows(); ++i)
@@ -622,21 +627,19 @@ namespace bgeot
 
   template<class MAT> inline void mat_out(std::ostream &o, const MAT &m)
   {
-    for (typename MAT::size_type l = 0; l < m.nrows(); l++ )
-    {
+    o << endl;
+    for (typename MAT::size_type l = 0; l < m.nrows(); l++ ) {
       for (typename MAT::size_type c = 0; c < m.ncols(); c++ )
 	o << m(l,c) << ' ';
-      o << '\n';
+      o << endl;
     }
   }
 
-  template<class T>  std::ostream &operator <<(std::ostream &o, const vsmatrix<T>& m)
-  { mat_out(o,m); return o; }
-
-  template<class T, int N>
-    std::ostream &operator <<(std::ostream &o, const fsmatrix<T,N>& m)
-  { mat_out(o,m); return o; }
+  template<class T>  std::ostream &operator <<
+  (std::ostream &o, const vsmatrix<T>& m) { mat_out(o,m); return o; }
 
+  template<class T, int N> std::ostream &operator <<
+  (std::ostream &o, const fsmatrix<T,N>& m) { mat_out(o,m); return o; }
 
   /* ******************************************************************** */
   /*	resolution d'un systeme lineaire par pivot de gauss.		  */
