@@ -10,26 +10,28 @@
 /*                                                                         */
 /* *********************************************************************** */
 /*                                                                         */
-/* Copyright (C) 2001  Yves Renard.                                        */
+/* Copyright (C) 1999-2001  Yves Renard.                                   */
 /*                                                                         */
 /* This file is a part of GETFEM++                                         */
 /*                                                                         */
 /* This program is free software; you can redistribute it and/or modify    */
-/* it under the terms of the GNU General Public License as published by    */
-/* the Free Software Foundation; version 2 of the License.                 */
+/* it under the terms of the GNU Lesser General Public License as          */
+/* published by the Free Software Foundation; version 2.1 of the License.  */
 /*                                                                         */
 /* This program is distributed in the hope that it will be useful,         */
 /* but WITHOUT ANY WARRANTY; without even the implied warranty of          */
 /* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
-/* GNU General Public License for more details.                            */
+/* GNU Lesser General Public License for more details.                     */
 /*                                                                         */
-/* You should have received a copy of the GNU General Public License       */
-/* along with this program; if not, write to the Free Software Foundation, */
-/* Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.         */
+/* You should have received a copy of the GNU Lesser General Public        */
+/* License along with this program; if not, write to the Free Software     */
+/* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,  */
+/* USA.                                                                    */
 /*                                                                         */
 /* *********************************************************************** */
 
 
+
 #ifndef __BGEOT_MESH_STRUCTURE_H
 #define __BGEOT_MESH_STRUCTURE_H
 
@@ -395,6 +397,35 @@ namespace bgeot
   }
 
 
+  /* ********************************************************************* */
+  /*                                                                       */
+  /*  Gives the list of edges of a mesh.                                   */
+  /*                                                                       */
+  /* ********************************************************************* */
+
+  struct edge_list_elt  {
+    size_type i, j;
+    size_type cv;
+    inline bool operator < (const edge_list_elt &e) const
+    {
+      if (i < e.i) return true; if (i > e.i) return false; 
+      if (j < e.j) return true; else if (j > e.j) return false;
+      if (cv < e.cv) return true; return false;
+    }
+    edge_list_elt(size_type ii, size_type jj, size_type ic = 0) : cv(ic)
+    { i = std::min(ii, jj); j = std::max(ii, jj); }
+    edge_list_elt(void) {}
+  };
+
+  typedef dal::dynamic_tree_sorted<edge_list_elt> edge_list;
+  
+
+  void mesh_edge_list_convex(const mesh_structure &m, size_type i, 
+			     edge_list &el, bool merge_convex = true);
+  void mesh_edges_list(const mesh_structure &m, edge_list &el, 
+		       bool merge_convex = true);
+
+
 
 }  /* end of namespace bgeot.                                              */
 
