@@ -10,22 +10,23 @@
 /*                                                                         */
 /* *********************************************************************** */
 /*                                                                         */
-/* Copyright (C) 2001  Yves Renard.                                        */
+/* Copyright (C) 2000-2002  Yves Renard.                                   */
 /*                                                                         */
 /* This file is a part of GETFEM++                                         */
 /*                                                                         */
 /* This program is free software; you can redistribute it and/or modify    */
-/* it under the terms of the GNU General Public License as published by    */
-/* the Free Software Foundation; version 2 of the License.                 */
+/* it under the terms of the GNU Lesser General Public License as          */
+/* published by the Free Software Foundation; version 2.1 of the License.  */
 /*                                                                         */
 /* This program is distributed in the hope that it will be useful,         */
 /* but WITHOUT ANY WARRANTY; without even the implied warranty of          */
 /* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
-/* GNU General Public License for more details.                            */
+/* GNU Lesser General Public License for more details.                     */
 /*                                                                         */
-/* You should have received a copy of the GNU General Public License       */
-/* along with this program; if not, write to the Free Software Foundation, */
-/* Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.         */
+/* You should have received a copy of the GNU Lesser General Public        */
+/* License along with this program; if not, write to the Free Software     */
+/* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,  */
+/* USA.                                                                    */
 /*                                                                         */
 /* *********************************************************************** */
 
@@ -112,7 +113,7 @@ namespace bgeot
 	    ++qv;
           #endif
 	}
-	return *(begin() + d);
+	return *(this->begin() + d);
       }
 
       template<class CONT> inline T& operator ()(const CONT &c)
@@ -123,7 +124,7 @@ namespace bgeot
 	for ( ; q != e; ++q, ++it) d += (*q) * (*it);
 	
 	if (d >= size()) DAL_THROW(std::out_of_range, "index out of range");
-	return *(begin() + d);
+	return *(this->begin() + d);
       }
 
       inline size_type size(void) const { return vsvector<T>::size(); }
@@ -182,7 +183,7 @@ namespace bgeot
     if (tmp->size() < dimt) tmp->resize(dimt);
     adjust_sizes(*mi);
     const_iterator pft = t.begin();
-    iterator pf = begin();
+    iterator pf = this->begin();
     size_type dd  =   coeff[ni]*(  sizes()[ni]-1)-1, co  =   coeff[ni];
     size_type ddt = t.coeff[ni]*(t.sizes()[ni]-1)-1, cot = t.coeff[ni];
     std::fill(mi->begin(), mi->end(), 0);
@@ -225,7 +226,7 @@ namespace bgeot
     if (tmp->size() < dimt) tmp->resize(dimt);
     adjust_sizes(*mi);
     const_iterator pft = t.begin();
-    iterator pf = begin();
+    iterator pf = this->begin();
     size_type dd  =   coeff[ni]*(  sizes()[ni]-1)-1, co  =   coeff[ni];
     size_type ddt = t.coeff[ni]*(t.sizes()[ni]-1)-1, cot = t.coeff[ni];
     std::fill(mi->begin(), mi->end(), 0);
