@@ -10,22 +10,23 @@
 /*                                                                         */
 /* *********************************************************************** */
 /*                                                                         */
-/* Copyright (C) 2001  Yves Renard.                                        */
+/* Copyright (C) 1995-2002  Yves Renard.                                   */
 /*                                                                         */
 /* This file is a part of GETFEM++                                         */
 /*                                                                         */
 /* This program is free software; you can redistribute it and/or modify    */
-/* it under the terms of the GNU General Public License as published by    */
-/* the Free Software Foundation; version 2 of the License.                 */
+/* it under the terms of the GNU Lesser General Public License as          */
+/* published by the Free Software Foundation; version 2.1 of the License.  */
 /*                                                                         */
 /* This program is distributed in the hope that it will be useful,         */
 /* but WITHOUT ANY WARRANTY; without even the implied warranty of          */
 /* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
-/* GNU General Public License for more details.                            */
+/* GNU Lesser General Public License for more details.                     */
 /*                                                                         */
-/* You should have received a copy of the GNU General Public License       */
-/* along with this program; if not, write to the Free Software Foundation, */
-/* Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.         */
+/* You should have received a copy of the GNU Lesser General Public        */
+/* License along with this program; if not, write to the Free Software     */
+/* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,  */
+/* USA.                                                                    */
 /*                                                                         */
 /* *********************************************************************** */
 
@@ -52,7 +53,7 @@ namespace bgeot
   { *result = *first; copy(++first, bgeot_count<N-1>(), ++result); }
 
   template <class InIter, class OutIter> inline
-    void copy(InIter first, bgeot_count<0>, OutIter result)
+    void copy(InIter, bgeot_count<0>, OutIter)
   { }
   
   template <int N, class OutIter, class T> inline
@@ -60,7 +61,7 @@ namespace bgeot
   { *first = a; fill(++first, bgeot_count<N-1>(), a); }
 
   template <class OutIter, class T> inline
-    void fill(OutIter first, bgeot_count<0>, const T& a)
+    void fill(OutIter , bgeot_count<0>, const T&)
   { }
 
   template <int N, class InIter, class OutIter, class T> inline
@@ -68,7 +69,7 @@ namespace bgeot
   { *result += *first * a; addmul(++first, bgeot_count<N-1>(), ++result, a); }
 
   template <class InIter, class OutIter, class T> inline 
-    void addmul(InIter first, bgeot_count<0>, OutIter result, const T& a)
+    void addmul(InIter, bgeot_count<0>, OutIter result, const T& a)
   { }
 
   template <int N, class OutIter, class T> inline
@@ -76,7 +77,7 @@ namespace bgeot
   { *first *= a; timeegal(++first, bgeot_count<N-1>(), a); }
 
   template <class OutIter, class T> inline
-    void timeegal(OutIter first, bgeot_count<0>, const T& a)
+    void timeegal(OutIter, bgeot_count<0>, const T&)
   { }
 
   template <int N, class OutIter, class T> inline
@@ -84,7 +85,7 @@ namespace bgeot
   { *first /= a; overegal(++first, bgeot_count<N-1>(), a); }
 
   template <class OutIter, class T> inline
-    void overegal(OutIter first, bgeot_count<0>, const T& a)
+    void overegal(OutIter, bgeot_count<0>, const T& a)
   { }
 
   template <int N, class InIter, class OutIter> inline
@@ -92,7 +93,7 @@ namespace bgeot
   { *result += *first; plusegal(++first, bgeot_count<N-1>(), ++result); }
 
   template <class InIter, class OutIter> inline 
-    void plusegal(InIter first, bgeot_count<0>, OutIter result)
+    void plusegal(InIter, bgeot_count<0>, OutIter)
   { }
 
   template <int N, class InIter, class OutIter> inline
@@ -100,7 +101,7 @@ namespace bgeot
   { *result -= *first; minusegal(++first, bgeot_count<N-1>(), ++result); }
 
   template <class InIter, class OutIter> inline 
-    void minusegal(InIter first, bgeot_count<0>, OutIter result)
+    void minusegal(InIter , bgeot_count<0>, OutIter)
   { }
 
   template <int N, class InIter, class OutIter> inline
@@ -111,7 +112,7 @@ namespace bgeot
   }
 
   template <class InIter, class OutIter> inline 
-    bool egalegal(InIter first, bgeot_count<0>, OutIter result)
+    bool egalegal(InIter, bgeot_count<0>, OutIter)
   { return true; }
 
   template <int N, class InIter, class OutIter> inline
@@ -122,7 +123,7 @@ namespace bgeot
   }
 
   template <class InIter, class OutIter> inline 
-    void vectsp(InIter first, bgeot_count<0>, OutIter result, double& res)
+    void vectsp(InIter, bgeot_count<0>, OutIter, double&)
   { }
 
 
@@ -309,7 +310,6 @@ namespace bgeot
    */
   template<class T> class vsvector : public std::vector<T>
   {
-   
     public:
 
       typedef typename std::vector<T>::size_type size_type;
@@ -319,14 +319,10 @@ namespace bgeot
       void out_of_range_error(void) const;
 
       #ifdef __GETFEM_VERIFY
-      inline const T& operator [](size_type l) const {
-	if (l >= size()) out_of_range_error();
-	return *(begin()+l);
-      }
-      inline T& operator [](size_type l) { 
-	if (l >= size()) out_of_range_error();
-	return *(begin()+l);
-      }
+      inline const T& operator [](size_type l) const
+      { if (l >= this->size()) out_of_range_error(); return *(this->begin()+l); }
+      inline T& operator [](size_type l)
+      { if (l >= this->size()) out_of_range_error(); return *(this->begin()+l); }
       #endif
 
       void fill(const T &);
@@ -342,14 +338,11 @@ namespace bgeot
       vsvector<T>& operator /=(const T &x);
     
       /// Constructor. For 2 components initialized with a0 and a1.
-      vsvector(T a0, T a1) : std::vector<T>(size_type(2))
-      { iterator p = begin(); *p++ = a0; *p++ = a1; }
+      vsvector(T a0, T a1);
       /// Constructor. For 3 components initialized with a0, a1 and a2.
-      vsvector(T a0, T a1, T a2) : std::vector<T>(size_type(3)) 
-      { iterator p = begin(); *p++ = a0; *p++ = a1; *p++ = a2;}
+      vsvector(T a0, T a1, T a2);
       /// Constructor. For 4 components initialized with a0, a1, a2 and a3.
-      vsvector(T a0, T a1, T a2, T a3) : std::vector<T>(size_type(4))
-      { iterator p = begin(); *p++ = a0; *p++ = a1; *p++ = a2; *p++ = a3;}
+      vsvector(T a0, T a1, T a2, T a3);
       /// Constructor. A vector with l components.
       vsvector(size_type l) : std::vector<T>(l) {}
       /// Constructor.
@@ -359,13 +352,24 @@ namespace bgeot
       { return std::vector<T>::capacity()*sizeof(T) + sizeof(vsvector<T>); }
   };
 
+  template<class T> vsvector<T>::vsvector(T a0, T a1)
+    : std::vector<T>(size_type(2))
+  { iterator p = this->begin(); *p++ = a0; *p++ = a1; }
+  template<class T> vsvector<T>::vsvector(T a0, T a1, T a2)
+    : std::vector<T>(size_type(3)) 
+  { iterator p = this->begin(); *p++ = a0; *p++ = a1; *p++ = a2;}
+  template<class T> vsvector<T>::vsvector(T a0, T a1, T a2, T a3)
+    : std::vector<T>(size_type(4))
+  { iterator p = this->begin(); *p++ = a0; *p++ = a1; *p++ = a2; *p++ = a3;}
+  
+
   template<class T>  void vsvector<T>::out_of_range_error(void) const
   { DAL_THROW(std::out_of_range, "out of range"); }
 
 
   template<class T>  void vsvector<T>::addmul(const T &a, const vsvector<T> &v)
   {                             
-    typename vsvector<T>::iterator d1 = begin(), e = end();
+    typename vsvector<T>::iterator d1 = this->begin(), e = this->end();
     const_iterator d2 = v.begin();
     if ( v.size() != this->size())
       DAL_THROW(dimension_error, "dimensions mismatch");
@@ -373,39 +377,36 @@ namespace bgeot
   }
 
   template<class T>  void vsvector<T>::fill(const T &x)
-  {
-    typename vsvector<T>::iterator d = begin(), e = end();
-    while (d != e) *d++ = x;
-  }
+  { std::fill(this->begin(), this->end(), x); }
  
   template<class T>  vsvector<T>& vsvector<T>::operator *=(const T &x)
   {
-    typename vsvector<T>::iterator d = begin(), e = end();
+    typename vsvector<T>::iterator d = this->begin(), e = this->end();
     while (d != e) *d++ *= x;
     return *this;
   }
 
   template<class T>  vsvector<T>& vsvector<T>::operator /=(const T &x)
   {
-    typename vsvector<T>::iterator d = begin(), e = end();
+    typename vsvector<T>::iterator d = this->begin(), e = this->end();
     while (d != e) *d++ /= x;
     return *this;
   }
 
   template<class T> vsvector<T>& vsvector<T>::operator +=(const vsvector<T>& w)
   {                             
-    typename vsvector<T>::iterator d1 = begin(), e = end();
+    typename vsvector<T>::iterator d1 = this->begin(), e = this->end();
     typename vsvector<T>::const_iterator d2 = w.begin();
-    if (size() != w.size()) DAL_THROW(dimension_error, "dimensions mismatch");
+    if (this->size() != w.size()) DAL_THROW(dimension_error, "dimensions mismatch");
     while (d1 != e) *d1++ += (*d2++);
     return *this;
   }
 
   template<class T> vsvector<T>& vsvector<T>::operator -=(const vsvector<T>& w)
   {    
-    typename vsvector<T>::iterator d1 = begin(), e = end();
+    typename vsvector<T>::iterator d1 = this->begin(), e = this->end();
     typename vsvector<T>::const_iterator d2 = w.begin();
-    if (size() != w.size()) DAL_THROW(dimension_error, "dimensions mismatch");
+    if (this->size() != w.size()) DAL_THROW(dimension_error, "dimensions mismatch");
 
     while (d1 != e) *d1++ -= (*d2++);
     return *this;
