@@ -1,3 +1,4 @@
+/* -*- c++ -*- (enables emacs c++ mode)                                    */
 /* *********************************************************************** */
 /*                                                                         */
 /* Library :  Dynamic Array Library (dal)                                  */
@@ -9,22 +10,23 @@
 /*                                                                         */
 /* *********************************************************************** */
 /*                                                                         */
-/* Copyright (C) 2001  Yves Renard.                                        */
+/* Copyright (C) 2000-2002  Yves Renard.                                   */
 /*                                                                         */
 /* This file is a part of GETFEM++                                         */
 /*                                                                         */
 /* This program is free software; you can redistribute it and/or modify    */
-/* it under the terms of the GNU General Public License as published by    */
-/* the Free Software Foundation; version 2 of the License.                 */
+/* it under the terms of the GNU Lesser General Public License as          */
+/* published by the Free Software Foundation; version 2.1 of the License.  */
 /*                                                                         */
 /* This program is distributed in the hope that it will be useful,         */
 /* but WITHOUT ANY WARRANTY; without even the implied warranty of          */
 /* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
-/* GNU General Public License for more details.                            */
+/* GNU Lesser General Public License for more details.                     */
 /*                                                                         */
-/* You should have received a copy of the GNU General Public License       */
-/* along with this program; if not, write to the Free Software Foundation, */
-/* Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.         */
+/* You should have received a copy of the GNU Lesser General Public        */
+/* License along with this program; if not, write to the Free Software     */
+/* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,  */
+/* USA.                                                                    */
 /*                                                                         */
 /* *********************************************************************** */
 
@@ -49,6 +51,10 @@ namespace dal
 
   template<class ITER> class tab_ref
   {
+    protected :
+
+      ITER _begin, _end;
+
     public :
 
       typedef typename std::iterator_traits<ITER>::value_type  value_type;
@@ -64,14 +70,8 @@ namespace dal
       typedef dal::reverse_iter<iterator> reverse_iterator;
       typedef size_t size_type;
     
-    protected :
-
-      ITER _begin, _end;
-
-    public :
-
       bool empty(void) const { return _begin == _end; }
-      
+      size_type size(void) const { return _end - _begin; }
 
       const iterator &begin(void) { return _begin; }
       const const_iterator &begin(void) const { return _begin; }
@@ -90,12 +90,12 @@ namespace dal
       const_reference back(void) const { return *(--(end())); }
       void pop_front(void) { ++_begin; }
 
-      const_reference operator [](size_type ii) const { return *(_begin + ii);}
+    //  changer en const_reference ?
+      value_type operator [](size_type ii) const { return *(_begin + ii);}
       reference operator [](size_type ii) { return *(_begin + ii); }
 
       tab_ref(void) {}
-      tab_ref(const ITER &b, const ITER &e) { _begin = b; _end = e; }
-
+      tab_ref(const ITER &b, const ITER &e) : _begin(b), _end(e) {}
   };
 
 
@@ -117,7 +117,7 @@ namespace dal
     typedef _tab_ref_index_iterator<ITER> iterator;
     
 
-    const ITER *piter;
+    ITER piter;
     
     iterator operator ++(int)
     { iterator tmp = *this; ++(*((_dnas_iterator *)(this))); return tmp; }
@@ -139,28 +139,26 @@ namespace dal
     { return *((_dnas_iterator *)(this)) - *((_dnas_iterator *)(&i)); }
 	
     reference operator *() const
-    { return *(*piter + *((*((_dnas_iterator *)(this))))); }
+    { return *(piter + *((*((_dnas_iterator *)(this))))); }
     reference operator [](int ii)
-    { return *(*piter + *((*((_dnas_iterator *)(this+ii))))); }
+    { return *(piter + *((*((_dnas_iterator *)(this+ii))))); }
     
     bool operator ==(const iterator &i) const
     { 
-      return ((*piter) == (*(i.piter))
+      return ((piter) == ((i.piter))
        && *((_dnas_iterator *)(this)) == *((*((_dnas_iterator *)(this)))));
     }
     bool operator !=(const iterator &i) const
     { return !(i == *this); }
     bool operator < (const iterator &i) const
     { 
-      return ((*piter) == (*(i.piter))
+      return ((piter) == ((i.piter))
 	 && *((_dnas_iterator *)(this)) < *((*((_dnas_iterator *)(this)))));
     }
 
     _tab_ref_index_iterator(void) {}
     _tab_ref_index_iterator(const ITER &iter, const _dnas_iterator &dnas_iter)
-	                                       : _dnas_iterator(dnas_iter)
-    { piter = &iter; }
-    
+      : _dnas_iterator(dnas_iter), piter(iter) {}
   };
 
 
@@ -189,7 +187,8 @@ namespace dal
     public :
 
       bool empty(void) const { return _index.empty(); }
-      
+      size_type size(void) const { return _index.size(); }
+
 
       iterator begin(void) { return iterator(_begin, _index.begin()); }
       const_iterator begin(void) const
@@ -213,8 +212,8 @@ namespace dal
       tab_ref_index(const ITER &b, const dynamic_array<size_type> &ind)
       { _begin = b; _index = ind; }
 
-
-      const_reference operator [](size_type ii) const
+    //  changer en const_reference ?
+      value_type operator [](size_type ii) const
       { return *(_begin + _index[ii]);}
       reference operator [](size_type ii) { return *(_begin + _index[ii]); }
 
@@ -237,7 +236,7 @@ namespace dal
       typedef _tab_ref_index_ref_iterator<ITER, ITER_INDEX> iterator;
       typedef size_t size_type;
 
-      const ITER *piter;
+      ITER piter;
       ITER_INDEX iter_index;
       
       iterator operator ++(int)
@@ -258,25 +257,20 @@ namespace dal
       { return iter_index - i.iter_index; }
 	
       reference operator *() const
-      {
-	// cout << "&iter = " << piter << endl;
-	// cout << "*iter_index = " << *iter_index << endl;
-	// cout << "*(*piter + *iter_index) = " << *(*piter + *iter_index) << endl;
-	return *(*piter + *iter_index);
-      }
+      { return *(piter + *iter_index); }
       reference operator [](int ii) const
-      { return *(*piter + *(iter_index+ii)); }
+      { return *(piter + *(iter_index+ii)); }
       
       bool operator ==(const iterator &i) const
-      { return ((*piter) == (*(i.piter)) && iter_index == i.iter_index); }
+      { return ((piter) == ((i.piter)) && iter_index == i.iter_index); }
       bool operator !=(const iterator &i) const { return !(i == *this); }
       bool operator < (const iterator &i) const
-      { return ((*piter) == (*(i.piter)) && iter_index < i.iter_index); }
+      { return ((piter) == ((i.piter)) && iter_index < i.iter_index); }
 
       _tab_ref_index_ref_iterator(void) {}
-      _tab_ref_index_ref_iterator(const ITER &iter,
+      _tab_ref_index_ref_iterator(const ITER &iter, 
 				  const ITER_INDEX &dnas_iter)
-      { piter = &iter; iter_index = dnas_iter; }
+	: piter(iter), iter_index(dnas_iter) {}
       
     };
 
@@ -306,6 +300,7 @@ namespace dal
     public :
 
       bool empty(void) const { return _index_begin == _index_end; }
+      size_type size(void) const { return _index_end - _index_begin; }
 
       iterator begin(void) { return iterator(_begin, _index_begin); }
       const_iterator begin(void) const
@@ -328,9 +323,10 @@ namespace dal
       tab_ref_index_ref(void) {}
       tab_ref_index_ref(const ITER &b, const ITER_INDEX &bi,
 			               const ITER_INDEX &ei)
-      { _begin = b; _index_begin = bi; _index_end = ei; }
+	: _begin(b), _index_begin(bi), _index_end(ei) {}
 
-      const_reference operator [](size_type ii) const
+    //  changer en const_reference ?
+      value_type operator [](size_type ii) const
       { return *(_begin + _index_begin[ii]);}
       reference operator [](size_type ii)
       { return *(_begin + _index_begin[ii]); }
@@ -342,41 +338,44 @@ namespace dal
   /* Reference on regularly spaced elements.                               */
   /* ********************************************************************* */
 
-  template<class ITER> struct _tab_ref_reg_spaced_iterator : public ITER
+  template<class ITER> struct _tab_ref_reg_spaced_iterator
   {
     typedef typename std::iterator_traits<ITER>::value_type value_type;
     typedef typename std::iterator_traits<ITER>::pointer    pointer;
     typedef typename std::iterator_traits<ITER>::reference  reference;
     typedef typename std::iterator_traits<ITER>::difference_type
                                                             difference_type;
+    typedef typename std::iterator_traits<ITER>::iterator_category
+                                                            iterator_category;
     typedef size_t size_type;
     typedef _tab_ref_reg_spaced_iterator<ITER> iterator;
+
+    ITER it;
     size_type N;
     
-    iterator operator ++(int)
-    { iterator tmp = *this; (*((ITER *)(this))) += N; return tmp; }
-    iterator operator --(int)
-    { iterator tmp = *this; (*((ITER *)(this))) -= N; return tmp; }
-    iterator &operator ++()
-    { (*((ITER *)(this))) += N; return *this; }
-    iterator &operator --()
-    { (*((ITER *)(this))) -= N; return *this; }
-    iterator &operator +=(difference_type i)
-    { (*((ITER *)(this))) += i * N; return *this; }
-    iterator &operator -=(difference_type i)
-    { (*((ITER *)(this))) -= i * N; return *this; }
-    iterator operator +(difference_type i) const
-    { iterator it = *this; return (it += i * N); }
+    iterator operator ++(int) { iterator tmp = *this; it += N; return tmp; }
+    iterator operator --(int) { iterator tmp = *this; it -= N; return tmp; }
+    iterator &operator ++()   { it += N; return *this; }
+    iterator &operator --()   { it -= N; return *this; }
+    iterator &operator +=(difference_type i) { it += i * N; return *this; }
+    iterator &operator -=(difference_type i) { it -= i * N; return *this; }
+    iterator operator +(difference_type i) const 
+    { iterator itt = *this; return (itt += i); }
     iterator operator -(difference_type i) const
-    { iterator it = *this; return (it -= i * N); }
+    { iterator itt = *this; return (itt -= i); }
     difference_type operator -(const iterator &i) const
-    { return (*((ITER *)(this)) - *((ITER *)(&i))) / N; }
-    
-    reference operator [](int ii) { return *(this+ii); }
-    
+    { return (it - i.it) / N; }
+
+    reference operator *() const { return *it; }
+    reference operator [](int ii) { return *(it + ii * N); }
+
+    bool operator ==(const iterator &i) const { return (it == i.it); }
+    bool operator !=(const iterator &i) const { return !(i == *this); }
+    bool operator < (const iterator &i) const { return (it < i.it); }
+
     _tab_ref_reg_spaced_iterator(void) {}
     _tab_ref_reg_spaced_iterator(const ITER &iter, size_type n)
-      : ITER(iter) { N = n; }
+      : it(iter), N(n) { }
     
   };
 
@@ -405,6 +404,7 @@ namespace dal
     public :
 
       bool empty(void) const { return _begin == _end; }
+      size_type size(void) const { return (_end - _begin) / N; }
 
       iterator begin(void) { return iterator(_begin, N); }
       const_iterator begin(void) const { return iterator(_begin, N); }
@@ -425,7 +425,7 @@ namespace dal
 
       tab_ref_reg_spaced(void) {}
       tab_ref_reg_spaced(const ITER &b, const ITER &e, size_type n)
-      { _begin = b; _index_begin = bi; _index_end = ei; N = n; }
+	: _begin(b), _end(e), N(n) {}
 
 
       const_reference operator [](size_type ii) const
@@ -448,19 +448,17 @@ namespace dal
                                                               difference_type;
     typedef std::forward_iterator_tag iterator_category;
     typedef _tab_ref_with_selection_iterator<ITER, COND> iterator;
-    const COND *cond;
+    const COND cond;
     
-    void forward(void) { while (!(*cond)(*this)) (*((ITER *)(this)))++; }
+    void forward(void) { while (!(cond)(*this)) ITER::operator ++(); }
     iterator &operator ++()
-    { (*((ITER *)(this)))++; forward(); return *this; }
+    { ITER::operator ++(); forward(); return *this; }
     iterator operator ++(int)
     { iterator tmp = *this; ++(*this); return tmp; }
-    void begin(const ITER &iter, const COND *c)
-    { *((ITER *)(this)) = iter; cond = c; forward(); }
     
     _tab_ref_with_selection_iterator(void) {}
-    _tab_ref_with_selection_iterator(const ITER &iter, const COND *c)
-      : ITER(iter) { cond = c; }
+    _tab_ref_with_selection_iterator(const ITER &iter, const COND c)
+      : ITER(iter), cond(c) {}
     
   };
 
@@ -484,15 +482,12 @@ namespace dal
 
     public :
 
-      iterator begin(void)
-      { iterator it; it.begin(_begin, &cond); return it; }
-      const_iterator begin(void) const { return iterator(_begin, &cond); }
-      iterator end(void) { return iterator(_end, &cond); }
-      const_iterator end(void) const { return iterator(_end, &cond); }
+      iterator begin(void) const
+      { iterator it(_begin, cond); it.forward(); return it; }
+      iterator end(void) const { return iterator(_end, cond); }
       bool empty(void) const { return _begin == _end; }
 
-      reference front(void) { return *begin(); }
-      const_reference front(void) const { return *begin(); }
+      reference front(void) const { return *begin(); }
       void pop_front(void) { ++_begin; _begin = begin(); }
 
       COND &condition(void) { return cond; }
