@@ -3,42 +3,38 @@
 /*                                                                         */
 /* Library :  Dynamic Array Library (dal)                                  */
 /* File    :  dal_std.h : Compatibility Header.                            */
-/*     									   */
 /*                                                                         */
 /* Date : June 01, 1995.                                                   */
 /* Author : Yves Renard, Yves.Renard@gmm.insa-tlse.fr                      */
 /*                                                                         */
 /* *********************************************************************** */
 /*                                                                         */
-/* Copyright (C) 2001  Yves Renard.                                        */
+/* Copyright (C) 1995-2002  Yves Renard.                                   */
 /*                                                                         */
 /* This file is a part of GETFEM++                                         */
 /*                                                                         */
 /* This program is free software; you can redistribute it and/or modify    */
-/* it under the terms of the GNU General Public License as published by    */
-/* the Free Software Foundation; version 2 of the License.                 */
+/* it under the terms of the GNU Lesser General Public License as          */
+/* published by the Free Software Foundation; version 2.1 of the License.  */
 /*                                                                         */
 /* This program is distributed in the hope that it will be useful,         */
 /* but WITHOUT ANY WARRANTY; without even the implied warranty of          */
 /* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
-/* GNU General Public License for more details.                            */
+/* GNU Lesser General Public License for more details.                     */
 /*                                                                         */
-/* You should have received a copy of the GNU General Public License       */
-/* along with this program; if not, write to the Free Software Foundation, */
-/* Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.         */
+/* You should have received a copy of the GNU Lesser General Public        */
+/* License along with this program; if not, write to the Free Software     */
+/* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,  */
+/* USA.                                                                    */
 /*                                                                         */
 /* *********************************************************************** */
 
-
 #ifndef __DAL_STD_H
 #define __DAL_STD_H
 
 #define __GETFEM_VERSION 1
 #define __GETFEM_REVISION 3
 
-#define __DAL_VERSION 1
-#define __DAL_REVISION 0
-
 #define __GETFEM_VERIFY
 
 #ifndef __USE_STD_IOSTREAM
@@ -48,7 +44,14 @@
 /* ********************************************************************** */
 /*	C++ Standard Headers.						  */
 /* ********************************************************************** */
-
+/*
+ essai rat pour sun CC 5.0 ...
+#if defined(__SUNPRO_CC) && __SUNPRO_CC >= 0x500
+# include <stdcomp.h>
+# undef _RWSTD_NO_CLASS_PARTIAL_SPEC
+# undef _RWSTD_NO_NAMESPACE
+#endif 
+*/
 #include <stdlib.h>
 #include <stddef.h>
 #include <math.h>
@@ -57,7 +60,7 @@
 #include <assert.h>
 #include <limits.h>
 #include <iostream>
-#include <ios>
+//#include <ios> essai
 #include <fstream>
 
 /* ********************************************************************** */
@@ -72,7 +75,19 @@
 #include <algorithm>
 #include <vector>
 #include <string>
-#include <sstream>
+#include <complex>
+
+#if defined(__GNUC__)
+#  if (__GNUC__ < 3)
+#    define USING_BROKEN_GCC295
+#    include <strstream>
+#    define stringstream strstream // not perfectly correct
+#  else
+#    include <sstream>
+#  endif
+#else
+#  include <sstream>
+#endif
 
 using std::endl;
 using std::cout;
@@ -177,6 +192,7 @@ namespace dal
 {
   template <class T> inline T sqr(T a) { return a * a; }
   template <class T> inline T abs(T a) { return (a < 0) ? -a : a; }
+  template <class T> inline T abs(std::complex<T> a) { return std::abs(a); }
   template <class T> inline T pos(T a) { return (a < 0) ? 0 : a; }
   template <class T> inline T neg(T a) { return (a < 0) ? -a : 0; }
 }
@@ -241,7 +257,7 @@ typedef unsigned char uint8_type;
 #if INT_MAX == 9223372036854775807L || INT_MAX == 9223372036854775807
   typedef signed int    int64_type;
   typedef unsigned int uint64_type;
-#elif LONG_MAX == 9223372036854775807L || INT_MAX == 9223372036854775807
+#elif LONG_MAX == 9223372036854775807L || LONG_MAX == 9223372036854775807
   typedef signed long int    int64_type;
   typedef unsigned long int uint64_type;
 #else
@@ -258,23 +274,54 @@ typedef unsigned char uint8_type;
   //    a += p[3]; a += i.p[3]; p[3] = a; a >>= 16;
   //  }
 
-    /* a continuer, passage au complement a 2 pour la soustraction        */
-    /* multiplication, transtypage, ...                                   */
-    /* a faire si besoin.                                                 */
-
   // };
 
   // struct uint64_type : int64_type
   // {
-    /* regle differente d'ecriture et de transformation en int ...        */
-
 
   // };
 
 #endif
+
+  /* juste pour forcer l'instanciation dans libgetfem de tous les
+     type qu'on est suceptible afficher. Si il en manque un (c'est le cas pour unsigned et float)
+     dans libgetfem.so qui est utilise dans libgetfemint.so alors on retrouve un warning
+     comme quoi le symbole __T_Q13std8ios_base et __T_Q13std11logic_error sont dupliqus
+     visiblement il reinstancie un peu trop de trucs .. 
+
+     pour s'assurer que le probleme n'est pas de retour, faire
+     nm libgetfem.so | grep 'td::basic_ostream<char, std::char_traits<char> >::operator <<'
+     et faire de mme dans tous les .o de matlabint, et s'assurer que tous ceux de matlabint
+     sont bien inclus dans libgetfem.so
+
+     enfin c'est pas tout a fait a puisque tous ces trucs sont deja instancies dans libcxxstd.a, et c'est le noeud du probleme j'imagine:
+     nm /usr/lib/cmplrs/cxx/V6.3-008/libcxxstd.a | grep 'td::basic_ostream<char, std::char_traits<char> >::operator <<' 
+     ---> std::basic_ostream<char, std::char_traits<char> >::operator <<(long long) | 0000000000006368 | T | 0000000000000008
+          std::basic_ostream<char, std::char_traits<char> >::operator <<(const void*) | 0000000000006368 | T | 0000000000000008
+	  std::basic_ostream<char, std::char_traits<char> >::operator <<(std::basic_ostream<char, std::char_traits<char> >& (*)(std::basic_ostream<char, std::char_traits<char> >&)) | 0000000000000000 | T | 0000000000000008
+	  std::basic_ostream<char, std::char_traits<char> >::operator <<(unsigned long long) | 0000000000006368 | T | 0000000000000008
+	  std::basic_ostream<char, std::char_traits<char> >::operator <<(unsigned int) | 0000000000006368 | T | 0000000000000008
+	  std::basic_ostream<char, std::char_traits<char> >::operator <<(unsigned long) | 0000000000006368 | T | 0000000000000008
+	  std::basic_ostream<char, std::char_traits<char> >::operator <<(unsigned short) | 0000000000006368 | T | 0000000000000008
+	  std::basic_ostream<char, std::char_traits<char> >::operator <<(double) | 0000000000006368 | T | 0000000000000008
+	  std::basic_ostream<char, std::char_traits<char> >::operator <<(float) | 0000000000006368 | T | 0000000000000008
+	  std::basic_ostream<char, std::char_traits<char> >::operator <<(int) | 0000000000006368 | T | 0000000000000008
+	  std::basic_ostream<char, std::char_traits<char> >::operator <<(bool) | 0000000000006368 | T | 0000000000000008
+	  std::basic_ostream<char, std::char_traits<char> >::operator <<(long double128) | 0000000000006368 | T | 0000000000000008
+	  std::basic_ostream<char, std::char_traits<char> >::operator <<(long) | 0000000000006368 | T | 0000000000000008
+	  std::basic_ostream<char, std::char_traits<char> >::operator <<(short) | 0000000000006368 | T | 0000000000000008
+	  std::basic_ostream<char, std::char_traits<char> >::operator <<(long double64) | 0000000000006368 | T | 0000000000000008
+  */
+  struct just_for_the_fine_cxx {
+    static void f() {
+      std::stringstream s; s << int(1) << double(2.0) 
+			     << "hello" << std::string("hello") << unsigned(1) << float(2) 
+			     << char('a') << (unsigned char)('b') << short(1) << (unsigned short)(2) 
+			     << long(1) << (unsigned long)(2) << (const void*)NULL;
+    }
+  };
   
 } /* end of namespace dal.                                                */
 
-
-
 #endif /* __DAL_STD_H */
+
