@@ -9,22 +9,23 @@
 /*                                                                         */
 /* *********************************************************************** */
 /*                                                                         */
-/* Copyright (C) 2001  Yves Renard.                                        */
+/* Copyright (C) 2001-2002  Yves Renard.                                   */
 /*                                                                         */
 /* This file is a part of GETFEM++                                         */
 /*                                                                         */
 /* This program is free software; you can redistribute it and/or modify    */
-/* it under the terms of the GNU General Public License as published by    */
-/* the Free Software Foundation; version 2 of the License.                 */
+/* it under the terms of the GNU Lesser General Public License as          */
+/* published by the Free Software Foundation; version 2.1 of the License.  */
 /*                                                                         */
 /* This program is distributed in the hope that it will be useful,         */
 /* but WITHOUT ANY WARRANTY; without even the implied warranty of          */
 /* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
-/* GNU General Public License for more details.                            */
+/* GNU Lesser General Public License for more details.                     */
 /*                                                                         */
-/* You should have received a copy of the GNU General Public License       */
-/* along with this program; if not, write to the Free Software Foundation, */
-/* Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.         */
+/* You should have received a copy of the GNU Lesser General Public        */
+/* License along with this program; if not, write to the Free Software     */
+/* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,  */
+/* USA.                                                                    */
 /*                                                                         */
 /* *********************************************************************** */
 
@@ -48,9 +49,10 @@ namespace getfem
 
   template<class VECT>
     void save_solution(const std::string &filename, mesh_fem &mf,
-		       const VECT &U, dim_type P, short_type K)
-  { 
+		       const VECT &U, short_type K)
+  { // a corriger
     dim_type N = mf.linked_mesh().dim();
+    dim_type P = mf.get_qdim();
     std::ofstream o(filename.c_str());
     if (!o) DAL_THROW(internal_error, "impossible to open file");
     dal::bit_vector nn = mf.convex_index();
@@ -90,11 +92,10 @@ namespace getfem
 	/* interpolation of the solution.                                  */
 	/* faux dans le cas des lments vectoriel.                        */
 	pt2 = pfe->node_of_dof(i);
+
 	for (size_type k = 0; k < P; ++k) {
-	  for (size_type j = 0; j < nbd1; ++j) {
-	    size_type dof1 = mf.ind_dof_of_element(cv)[j];
-	    coeff[j] = U[dof1*P+k];
-	  }
+	  for (size_type j = 0; j < nbd1 / P; ++j)
+	    coeff[j] = U[mf.ind_dof_of_element(cv)[j*P+k]];
 	  // il faudrait utiliser les fem_precomp pour accelerer.
 	  pf1->interpolation(pt2, G, pgt, coeff, val);
 	  pt3[k] = val[0];
@@ -111,7 +112,7 @@ namespace getfem
 
   template<class VECT>
     void load_solution(const std::string &fi, getfem_mesh &mesh,
-		       mesh_fem &mef, VECT &U, dim_type &P, short_type &K) {
+		       mesh_fem &mef, VECT &U, short_type &K) {
     std::ifstream ist(fi.c_str());
     size_type token, DIM, Np, N;
     char tmp[100], c;
@@ -119,6 +120,7 @@ namespace getfem
     dal::dynamic_array<base_node> ptab;
     dal::dynamic_array<scalar_type> vtab;
     size_type nbvtab = 0;
+    dim_type P;
     
     if (!ist) DAL_THROW(file_not_found_error, "File " << fi << " not found");
 
@@ -145,7 +147,7 @@ namespace getfem
 	  // cout << "token " << token << " val = " << k << endl;
 	  switch (token) {
 	  case 1 : DIM = N = k; if (k == 0) error = true; break;
-	  case 2 : P = k; if (k == 0) error = true; break;
+	  case 2 : P = k; mef.set_qdim(P); if (k == 0) error = true; break;
 	  case 3 : K = k; break;
 	  case 4 : DIM = k; break;
 	  }
@@ -217,22 +219,23 @@ namespace getfem
     if (error) DAL_THROW(failure_error, "Format error in file " << fi);
     
     // Data repartition
+    //  corriger
     
     dal::bit_vector nn = mesh.convex_index();
-    U.resize(mef.nb_dof() * P);
+    U.resize(mef.nb_dof());
     std::fill(U.begin(), U.end(), 0.0);
-    std::vector<int> cp(mef.nb_dof());
+    std::vector<int> cp(mef.nb_dof() / P);
     std::fill(cp.begin(), cp.end(), 0);
     size_type l = 0, i;
     for (i << nn; i != size_type(-1); i << nn) {
       size_type nbd = mef.nb_dof_of_element(i);
-      for (size_type j = 0; j < nbd; ++j, ++l) {
-	size_type dof = mef.ind_dof_of_element(i)[j];
-	for (short_type k = 0; k < P; ++k) U[dof*P +k] += vtab[l*P+k];
+      for (size_type j = 0; j < nbd / P; ++j, ++l) {
+	size_type dof = mef.ind_dof_of_element(i)[j*P];
+	for (short_type k = 0; k < P; ++k) U[dof +k] += vtab[l*P+k];
 	(cp[dof])++;
       }
     }
-    for (i = 0; i < mef.nb_dof(); ++i) {
+    for (i = 0; i < mef.nb_dof() / P; ++i) {
       if (cp[i] == 0) DAL_THROW(internal_error, "Internal error");
       for (short_type k = 0; k < P; ++k)
 	U[i*P +k] /= getfem::scalar_type(cp[i]);
@@ -240,37 +243,6 @@ namespace getfem
     
   }
 
-  /* ********************************************************************* */
-  /*                                                                       */
-  /*  Gives the list of edges of a mesh.                                   */
-  /*                                                                       */
-  /* ********************************************************************* */
-
-  struct edge_list_elt  {
-    size_type i, j;
-    size_type cv;
-    inline bool operator < (const edge_list_elt &e) const
-    {
-      if (i < e.i) return true; if (i > e.i) return false; 
-      if (j < e.j) return true; else if (j > e.j) return false;
-      if (cv < e.cv) return true; return false;
-    }
-    edge_list_elt(size_type ii, size_type jj, size_type ic = 0) : cv(ic)
-    { i = std::min(ii, jj); j = std::max(ii, jj); }
-    edge_list_elt(void) {}
-  };
-
-  typedef dal::dynamic_tree_sorted<edge_list_elt> edge_list;
-  
-
-  void mesh_edge_list_convex(const getfem_mesh &m, size_type i, 
-			     edge_list &el, bool merge_convex = true);
-  void mesh_edges_list(const getfem_mesh &m, edge_list &el, 
-		       bool merge_convex = true);
-
-
-
-
   /* ********************************************************************* */
   /*                                                                       */
   /*  interpolation of a solution on same mesh.                            */
@@ -278,6 +250,167 @@ namespace getfem
   /*                                                                       */
   /* ********************************************************************* */
 
+  template<class VECT>
+    void interpolation_solution_same_mesh(mesh_fem &mf_source, mesh_fem &mf_target,
+					  const VECT &U, VECT &V)
+  {
+    dal::bit_vector nn = mf_source.convex_index();
+    size_type cv;
+    base_node val(1);
+    base_matrix G;
+    base_vector coeff;
+    size_type qdim = mf_source.get_qdim();
+    if ( &(mf_source.linked_mesh()) != &(mf_target.linked_mesh()))
+      DAL_THROW(failure_error, "Meshes should be the same in this function.");
+
+    if (qdim != mf_target.get_qdim())
+      DAL_THROW(failure_error, "Attempt to interpolate a field of dimension " << 
+		qdim << " on a mesh_fem whose Qdim is " << 
+		int(mf_target.get_qdim()));
+
+    for (cv << nn; cv != ST_NIL; cv << nn) {
+      bgeot::pgeometric_trans pgt = mf_source.linked_mesh().trans_of_convex(cv);
+      pfem pf_s = mf_source.fem_of_element(cv);
+      pfem pf_t = mf_target.fem_of_element(cv);
+      size_type nbd_s = pf_s->nb_dof();
+      size_type nbd_t = pf_t->nb_dof();
+      coeff.resize(nbd_s);
+
+      if (!(pf_s->is_equivalent())) 
+	transfert_to_G(G, mf_source.linked_mesh().points_of_convex(cv));
+
+      if (pf_s->target_dim() != 1 || pf_t->target_dim() != 1)
+	DAL_THROW(to_be_done_error, "vector FEM interpolation still to be done ... ");
+      
+      for (size_type i = 0; i < nbd_t; ++i) {
+	size_type dof_t = mf_target.ind_dof_of_element(cv)[i*qdim];
+	/* interpolation of the solution.                                  */
+	/* faux dans le cas des lments vectoriel.                        */
+	for (size_type k = 0; k < qdim; ++k) {
+	  for (size_type j = 0; j < nbd_s; ++j) {
+	    size_type dof_s = mf_source.ind_dof_of_element(cv)[j*qdim+k];
+	    coeff[j] = U[dof_s];
+	  }
+	  // il faudrait utiliser les fem_precomp pour accelerer.
+	  pf_s->interpolation(pf_t->node_of_dof(i), G, pgt, coeff, val);
+	  V[dof_t + k] = val[0];
+	}
+      }
+    }
+  }
+
+  /* ********************************************************************* */
+  /*                                                                       */
+  /*  interpolation of a solution on another mesh.                         */
+  /*   - mf_target must be of lagrange type.                               */
+  /*   - the solution must be continuous.                                  */
+  /*                                                                       */
+  /* ********************************************************************* */
+
+
+
+  template<class VECT>
+    void interpolation_solution(mesh_fem &mf_source, mesh_fem &mf_target,
+				const VECT &U, VECT &V)
+  {
+    size_type cv;
+    base_node val(1);
+    bgeot::geotrans_inv gti;
+    dal::dynamic_array<base_node> ptab;
+    dal::dynamic_array<size_type> itab;
+    base_vector coeff;
+    base_matrix G;
+
+    
+    if (&mf_source.linked_mesh() == &mf_target.linked_mesh()) {
+      interpolation_solution_same_mesh(mf_source, mf_target, U, V);
+      return;
+    }
+    size_type qdim = mf_source.get_qdim();
+    if (qdim != mf_target.get_qdim())
+      DAL_THROW(failure_error, "Attempt to interpolate a field of dimension " << 
+		qdim << " on a mesh_fem whose Qdim is " << 
+		int(mf_target.get_qdim()));
+
+    dal::bit_vector tdof_added; 
+
+    /* stocke des paires <indice, repetition> */
+    std::deque<std::pair<size_type,size_type> > gti_pt_2_target_dof;
+
+    /* stockage de tous les points dans le gti,
+       et remplissage du tableau gti_pt_2_target_dof pour
+       pouvoir retrouver le numro du ddl  partir de son indice
+       dans la liste de points de gti 
+    */
+    for (dal::bit_vector::const_iterator it = mf_target.convex_index().begin();
+	 it != mf_target.convex_index().end(); ++it) {
+      if (*it) {
+	pfem pf_t = mf_target.fem_of_element(it.index());
+	size_type qmult = mf_target.get_qdim() / pf_t->target_dim();
+	
+	if (pf_t->target_dim() != 1) DAL_THROW(failure_error, "still some work to do on vector FEMs!");
+
+	for (size_type j=0; j < pf_t->nb_dof(); ++j) {
+	  size_type dof_t = mf_target.ind_dof_of_element(it.index())[j*qmult];
+	  if (!tdof_added[dof_t]) {
+	    gti.add_point(mf_target.point_of_dof(dof_t));
+	    gti_pt_2_target_dof.push_back(std::pair<size_type,size_type>(dof_t,qmult));
+	    tdof_added.add(dof_t);
+	  }
+	}
+      }
+    }
+    // il faudrait controler que tous les ddl de mf_target sont de
+    // type lagrange
+    dal::bit_vector nn = mf_source.convex_index(), ddl_touched;
+    ddl_touched.add(0, mf_target.nb_dof()-1);
+
+    for (cv << nn; cv != ST_NIL; cv << nn)
+    {
+      bgeot::pgeometric_trans pgt = mf_source.linked_mesh().trans_of_convex(cv);
+      //cout << "dealing with convex " << cv << " remaining " << nn.card() << endl;
+      size_type nb = gti.points_in_convex(mf_source.linked_mesh().convex(cv),
+					  pgt, ptab, itab);
+      //cout << "nb points in this convex " << nb << endl;
+      // cout << "convex : " << mf_source.linked_mesh().convex(cv) << endl;
+      pfem pf_s = mf_source.fem_of_element(cv);
+      if (!(pf_s->is_equivalent())) 
+	transfert_to_G(G, mf_source.linked_mesh().points_of_convex(cv));
+      size_type nbd_s = pf_s->nb_dof();
+      coeff.resize(nbd_s);
+      for (size_type i = 0; i < nb; ++i)
+      {
+	size_type dof_t = gti_pt_2_target_dof[itab[i]].first;
+	size_type nrep  = gti_pt_2_target_dof[itab[i]].second;
+	assert(nrep>0);
+	// cout << "dealing with ddl : " << itab[i] << "  coords : " << mf_target.point_of_dof(itab[i]) << " internal coords : " << ptab[i] << endl;
+	if (ddl_touched[dof_t])
+	{ // inverser les deux boucles pour gagner du temps ?
+	  // Il faut verifier que le ddl est bien de Lagrange ...
+	  for (size_type k = 0; k < nrep; ++k) {
+	    for (size_type j = 0; j < nbd_s; ++j) {
+	      size_type dof_s = mf_source.ind_dof_of_element(cv)[j*nrep+k];
+	      coeff[j] = U[dof_s];
+	      //cerr << "dof_s=" << dof_s << " dof_t=" << dof_t << " k=" << k << " i=" << i << endl;
+	    }
+	    pf_s->interpolation(ptab[i], G, pgt, coeff, val);
+	    V[dof_t + k] = val[0];
+	    ddl_touched.sup(dof_t+k);
+	  }
+	}
+      }
+    }
+    // cout << "ddl untouched : " << ddl_touched << endl;
+    if (ddl_touched.card() != 0)
+      cerr << "WARNING : in interpolation_solution,"
+	   << " all points have not been touched" << endl;
+  }
+
+
+
+  /*
+    OLD INTERPOLATION 
+  */
   template<class VECT>
     void interpolation_solution_same_mesh(mesh_fem &mf, mesh_fem &mf_target,
 					  const VECT &U, VECT &V, dim_type P)
@@ -307,7 +440,7 @@ namespace getfem
 	DAL_THROW(to_be_done_error, "to be done ... ");
       
       for (size_type i = 0; i < nbd2; ++i) {
-	size_type dof1 = mf_target.ind_dof_of_element(cv)[i];
+	size_type dof2 = mf_target.ind_dof_of_element(cv)[i];
 	/* interpolation of the solution.                                  */
 	/* faux dans le cas des lments vectoriel.                        */
 	pt2 = pfe->node_of_dof(i);
@@ -318,7 +451,7 @@ namespace getfem
 	  }
 	  // il faudrait utiliser les fem_precomp pour accelerer.
 	  pf1->interpolation(pt2, G, pgt, coeff, val);
-	  V[dof1*P + k] = val[0];
+	  V[dof2*P + k] = val[0];
 	}
       }
     }
@@ -364,10 +497,10 @@ namespace getfem
     for (cv << nn; cv != ST_NIL; cv << nn)
     {
       bgeot::pgeometric_trans pgt = mf.linked_mesh().trans_of_convex(cv);
-      cout << "dealing with convex " << cv << " remaining " << nn.card() << endl;
+      //cout << "dealing with convex " << cv << " remaining " << nn.card() << endl;
       nb = gti.points_in_convex(mf.linked_mesh().convex(cv),
 			    mf.linked_mesh().trans_of_convex(cv), ptab, itab);
-      cout << "nb points in this convex " << nb << endl;
+      //cout << "nb points in this convex " << nb << endl;
       // cout << "convex : " << mf.linked_mesh().convex(cv) << endl;
       pfem pfe = mf.fem_of_element(cv);
       if (!(pfe->is_equivalent())) 
