@@ -9,22 +9,23 @@
 /*                                                                         */
 /* *********************************************************************** */
 /*                                                                         */
-/* Copyright (C) 2001  Yves Renard.                                        */
+/* Copyright (C) 1999-2002  Yves Renard.                                   */
 /*                                                                         */
 /* This file is a part of GETFEM++                                         */
 /*                                                                         */
 /* This program is free software; you can redistribute it and/or modify    */
-/* it under the terms of the GNU General Public License as published by    */
-/* the Free Software Foundation; version 2 of the License.                 */
+/* it under the terms of the GNU Lesser General Public License as          */
+/* published by the Free Software Foundation; version 2.1 of the License.  */
 /*                                                                         */
 /* This program is distributed in the hope that it will be useful,         */
 /* but WITHOUT ANY WARRANTY; without even the implied warranty of          */
 /* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
-/* GNU General Public License for more details.                            */
+/* GNU Lesser General Public License for more details.                     */
 /*                                                                         */
-/* You should have received a copy of the GNU General Public License       */
-/* along with this program; if not, write to the Free Software Foundation, */
-/* Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.         */
+/* You should have received a copy of the GNU Lesser General Public        */
+/* License along with this program; if not, write to the Free Software     */
+/* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,  */
+/* USA.                                                                    */
 /*                                                                         */
 /* *********************************************************************** */
 
@@ -64,6 +65,8 @@ namespace getfem
   pdof_description already_numerate_dof(dim_type);
   /// Product description of the descriptions *pnd1 and *pnd2.
   pdof_description product_dof(pdof_description, pdof_description);
+
+  pdof_description to_coord_dof(pdof_description p, dim_type ct);
   
   /** Gives a total order on the dof description compatible with the
    *   identification.
@@ -73,6 +76,7 @@ namespace getfem
   bool dof_linkable(pdof_description);
   /// Says if the two dofs can be identified.
   bool dof_compatibility(pdof_description, pdof_description);
+
   
   /* ******************************************************************** */
   /*	Classes for description of a finite element.                      */
@@ -234,9 +238,23 @@ namespace getfem
     const std::vector<FUNC> &base(void) const { return _base; }
     std::vector<FUNC> &base(void) { return _base; }
     
+    /* just to please for HP aCC and SGI CC */
+    virtual void interpolation(pfem_precomp pfp, size_type ii,
+			       const base_matrix &G,
+			       bgeot::pgeometric_trans pgt, 
+      const base_vector coeff, base_node &val) const { 
+      virtual_fem::interpolation(pfp,ii,G,pgt,coeff,val); 
+    }
     void interpolation(const base_node &x, const base_matrix &G, 
 		       bgeot::pgeometric_trans pgt,
 		       const base_vector coeff, base_node &val) const;
+    void interpolation_grad(pfem_precomp pfp, size_type ii,
+			    const base_matrix &G,
+			    bgeot::pgeometric_trans pgt, 
+			    const base_vector coeff,
+			    base_matrix &val) const { 
+      virtual_fem::interpolation_grad(pfp,ii,G,pgt,coeff,val); 
+    }
     void interpolation_grad(const base_node &x, const base_matrix &G,
 			    bgeot::pgeometric_trans pgt,
 			    const base_vector coeff, base_matrix &val) const;
