@@ -1,7 +1,7 @@
+$bin_dir = "$ENV{srcdir}/../bin";
+$tmp = `$bin_dir/createmp laplacian.param`;
 
-$tmp = `../bin/createmp laplacian.param`;
-# print "TMP = $tmp\n";
-sub catch { `rm -f $tmp`; }
+sub catch { `rm -f $tmp`; exit(1); }
 $SIG{INT} = 'catch';
 
 open(TMPF, ">$tmp") or die "Open file impossible : $!\n";
