@@ -1,7 +1,9 @@
+$bin_dir = "$ENV{srcdir}/../bin";
 
-$tmp = `../bin/createmp laplacian.param`;
+
+$tmp = `$bin_dir/createmp test.param`;
 # print "TMP = $tmp\n";
-sub catch { `rm -f $tmp`; }
+sub catch { `rm -f $tmp`; exit(1); }
 $SIG{INT} = 'catch';
 
 open(TMPF, ">$tmp") or die "Open file impossible : $!\n";
